//
// Created by tim on 05.04.23.
//

#include <fstream>
#include "CTUApexConsumer.h"
#include "Util.h"
#include "llvm/Demangle/Demangle.h"

std::string CTUApexConsumer::outline(metacg::Callgraph &cg) {
    SPDLOG_INFO("Compute set of functions to be extracted");
    std::unordered_set<clang::Decl *> availableFunctionDecls;
    for (const auto &node: cg.getNodes()) {
        if (node.second->has<ExtractionMetadata>() && node.second->get<ExtractionMetadata>()->isMarkedToExtract()) {
            if (node.second->get<ExtractionMetadata>()->get_decl_node() == nullptr) {
                std::cout << "node " << node.second->getFunctionName()
                          << " was marked to extract but is not part of this TU\n";
            } else {
                auto functionDecl = node.second->get<ExtractionMetadata>()->get_decl_node();
                availableFunctionDecls.insert(functionDecl);
                addCalleesToSet(availableFunctionDecls, node.first, cg);
            }
        }
    }

    //Fixme: does this need to extract cross TU as well?
    SPDLOG_INFO("Compute set of used globals");
    std::unordered_set<clang::Decl *> availableGlobals;
    for (const auto &node: availableFunctionDecls) {
        assert(clang::isa<clang::FunctionDecl>(node) &&
               "Extractionset should only contain function decls at this point");
        GlobalsMarker gMarker;
        auto func_set=gMarker.get_nodes_to_outline(clang::cast<clang::FunctionDecl>(node));
        availableGlobals.insert(func_set.begin(),func_set.end());
    }

    std::string returnString;

    for (const auto &decl: availableGlobals) {
        clang::SourceRange sourceRange = decl->getSourceRange();
        returnString += clang::Lexer::getSourceText(clang::CharSourceRange::getTokenRange(sourceRange),
                                                    decl->getASTContext().getSourceManager(), clang::LangOptions(),
                                                    nullptr).str();
        returnString += "\n";
    }

    for (const auto &decl: availableFunctionDecls) {
        clang::SourceRange sourceRange = decl->getSourceRange();
        returnString += clang::Lexer::getSourceText(clang::CharSourceRange::getTokenRange(sourceRange),
                                                    decl->getASTContext().getSourceManager(), clang::LangOptions(),
                                                    nullptr).str();
        returnString += "\n";
    }

    return returnString;
}


void CTUApexConsumer::HandleTranslationUnit(clang::ASTContext &Context) {
    const auto& translation_unit = Context.getTranslationUnitDecl();
    SPDLOG_INFO("Mapping functions to the call graph");
    FunctionFinderVisitor functionFinderVisitor(call_graph);
    functionFinderVisitor.TraverseTranslationUnitDecl(translation_unit);

    SPDLOG_INFO("Generate code skeleton");

    auto t=outline(*call_graph);

    const auto& sm = Context.getSourceManager();
    std::string filename = sm.getFileEntryForID(sm.getMainFileID())->getName().str();
    std::ofstream outFile;
    outFile.open ("mini"+filename);
    outFile << col2str(includes, "#include \"", "\"\n#include \"", "\"\n") << t << "\n";;
    outFile.close();

    SPDLOG_INFO("Generate wrapper calls");

    // Outline the code we want
    //SPDLOG_DEBUG("Generating outline");
    //assert(output.empty());
    //ASTOutliner outliner(&context, dump_anal_res);
    //auto extracted_source = outliner.generate_outline(marked, only_wrapper);

    //if (!only_wrapper) {
    //    auto main_source = outliner.generate_main_function(marked);
    //    output = extracted_source + main_source;
    //} else {
    //    output = extracted_source;
    //}

    SPDLOG_INFO("Clean CG for next run");
    for(const auto& node: call_graph->getNodes()){
        if(node.second->has<ExtractionMetadata>()){
            node.second->get<ExtractionMetadata>()->set_decl_node(nullptr);
        }
    }
}

