//
// Created by tim on 05.04.23.
//

#include "FunctionFinderVisitor.h"
#include "ExtractionMetadata.h"
#include "Util.h"

#include <iostream>
#include <clang/AST/Mangle.h>

std::vector<std::string> getMangledName(clang::NamedDecl const *const nd) {
    if (!nd) {
        SPDLOG_ERROR("NamedDecl was nullptr");
        assert(nd && "NamedDecl and MangleContext must not be nullptr");
        return {"__NO_NAME__"};
    }

    clang::ASTNameGenerator NG(nd->getASTContext());

    if (llvm::isa<clang::CXXRecordDecl>(nd) || llvm::isa<clang::CXXMethodDecl>(nd) ||
        llvm::isa<clang::ObjCInterfaceDecl>(nd) || llvm::isa<clang::ObjCImplementationDecl>(nd)) {
        return NG.getAllManglings(nd);
    }

    return {NG.getName(nd)};
}

bool FunctionFinderVisitor::VisitFunctionDecl(clang::FunctionDecl *functionDecl) {
    if(functionDecl->isDependentContext()){
        //SPDLOG_TRACE("Function {} is context dependend\n",functionDecl->getNameAsString());
        return true;
    }

    const auto possibleNames = getMangledName(functionDecl);

    //std::cout<<functionDecl->getNameAsString()<<col2str(possibleNames," All possible manglings are:(",",",")")<<"\n";

    for(const auto& name : possibleNames){
        // We should only override if we have a body or if it is null
        if(functionDecl->getASTContext().getSourceManager().isInMainFile(functionDecl->getSourceRange().getBegin())){
            //SPDLOG_DEBUG("Function is part of main file\n");
        }else{
            //SPDLOG_DEBUG("Function is from header\n");
            continue;
        }

        if(!callgraph->hasNode(name)){ // if we do not have the node, it is not used in the program and can be ignored
            //std::cout<<"Node "<<name <<" is not in the graph\n";
            continue;
        }

        //SPDLOG_TRACE("Attaching AST node: {}, to the call graph",possibleNames.at(0));
        if (functionDecl->hasBody() || callgraph->getNode(name)->getOrCreateMD<ExtractionMetadata>()->get_decl_node()==
                                       nullptr) {
            callgraph->getNode(name)->getOrCreateMD<ExtractionMetadata>()->set_decl_node(functionDecl);
        }
    }
    return true;
};

