//
// Created by tim on 05.04.23.
//

#include "CallSites.h"
#include <fstream>
#include "Util.h"
#include "spdlog/spdlog.h"

std::vector<CallSite> CallSite::read_callsites_from_file(const std::string &file) {
    std::vector<CallSite> result;

    std::ifstream ifstream();
    std::ifstream ifs(file, std::ifstream::in);
    if (ifs.is_open()) {
        std::string line;
        while (std::getline(ifs, line)) {
            // remove leading and trailing whitespace
            line = trim(line);

            size_t first_delimiter = line.find_first_of(':');
            size_t second_delimiter = line.find_last_of(':');

            // figleaf syntax checks
            if (first_delimiter == std::string::npos || second_delimiter == std::string::npos || first_delimiter == 0 ||
                second_delimiter == line.size() - 1 || first_delimiter >= second_delimiter - 1) {
                SPDLOG_ERROR("Malformed call site specification");
                break;
            }

            std::string file_name = line.substr(0, first_delimiter);

            std::string line_number_str = line.substr(first_delimiter + 1, second_delimiter - first_delimiter - 1);
            unsigned int line_number = std::numeric_limits<unsigned int>::max();
            try {
                line_number = std::stoi(line_number_str);
            } catch (...) {
                SPDLOG_ERROR("Invalid line number in call site specification: " + line_number_str);
                break;
            }

            std::string column_number_str = line.substr(second_delimiter + 1);
            unsigned int column_number = std::numeric_limits<unsigned int>::max();
            try {
                column_number = std::stoi(column_number_str);
            } catch (...) {
                SPDLOG_ERROR("Invalid column number in call site specification: " + column_number_str);
                break;
            }

            result.emplace_back(file_name, line_number, column_number);
        }
        ifs.close();
    } else {
        SPDLOG_ERROR("Could not open file to read call sites");
    }

    return result;
}